'use strict';

;(function () {
  var store = {},
      win = window,
      doc = win.document,
      localStorageName = 'localStorage',
      namespace = '__storejs__',
      storage;

  store.disabled = false;
  store.set = function (key, value) {};
  store.get = function (key) {};
  store.remove = function (key) {};
  store.clear = function () {};
  store.transact = function (key, defaultVal, transactionFn) {
    var val = store.get(key);
    if (transactionFn == null) {
      transactionFn = defaultVal;
      defaultVal = null;
    }
    if (typeof val == 'undefined') {
      val = defaultVal || {};
    }
    transactionFn(val);
    store.set(key, val);
  };
  store.getAll = function () {};

  store.serialize = function (value) {
    return JSON.stringify(value);
  };
  store.deserialize = function (value) {
    if (typeof value != 'string') {
      return undefined;
    }
    try {
      return JSON.parse(value);
    } catch (e) {
      return value || undefined;
    }
  };

  // Functions to encapsulate questionable FireFox 3.6.13 behavior
  // when about.config::dom.storage.enabled === false
  // See https://github.com/marcuswestin/store.js/issues#issue/13
  function isLocalStorageNameSupported() {
    try {
      return localStorageName in win && win[localStorageName];
    } catch (err) {
      return false;
    }
  }

  if (isLocalStorageNameSupported()) {
    storage = win[localStorageName];
    store.set = function (key, val) {
      if (val === undefined) {
        return store.remove(key);
      }
      storage.setItem(key, store.serialize(val));
      return val;
    };
    store.get = function (key) {
      return store.deserialize(storage.getItem(key));
    };
    store.remove = function (key) {
      storage.removeItem(key);
    };
    store.clear = function () {
      storage.clear();
    };
    store.getAll = function () {
      var ret = {};
      for (var i = 0; i < storage.length; ++i) {
        var key = storage.key(i);
        ret[key] = store.get(key);
      }
      return ret;
    };
  } else if (doc.documentElement.addBehavior) {
    var storageOwner, storageContainer;
    var forbiddenCharsRegex;

    (function () {
      var withIEStorage = function withIEStorage(storeFunction) {
        return function () {
          var args = Array.prototype.slice.call(arguments, 0);
          args.unshift(storage);
          // See http://msdn.microsoft.com/en-us/library/ms531081(v=VS.85).aspx
          // and http://msdn.microsoft.com/en-us/library/ms531424(v=VS.85).aspx
          storageOwner.appendChild(storage);
          storage.addBehavior('#default#userData');
          storage.load(localStorageName);
          var result = storeFunction.apply(store, args);
          storageOwner.removeChild(storage);
          return result;
        };
      }

      // In IE7, keys may not contain special chars. See all of https://github.com/marcuswestin/store.js/issues/40
      ;

      var ieKeyFix = function ieKeyFix(key) {
        return key.replace(forbiddenCharsRegex, '___');
      };

      // Since #userData storage applies only to specific paths, we need to
      // somehow link our data to a specific path.  We choose /favicon.ico
      // as a pretty safe option, since all browsers already make a request to
      // this URL anyway and being a 404 will not hurt us here.  We wrap an
      // iframe pointing to the favicon in an ActiveXObject(htmlfile) object
      // (see: http://msdn.microsoft.com/en-us/library/aa752574(v=VS.85).aspx)
      // since the iframe access rules appear to allow direct access and
      // manipulation of the document element, even for a 404 page.  This
      // document can be used instead of the current document (which would
      // have been limited to the current path) to perform #userData storage.
      try {
        storageContainer = new ActiveXObject('htmlfile');
        storageContainer.open();
        storageContainer.write('<s' + 'cript>document.w=window</s' + 'cript><iframe src="/favicon.ico"></iframe>');
        storageContainer.close();
        storageOwner = storageContainer.w.frames[0].document;
        storage = storageOwner.createElement('div');
      } catch (e) {
        // somehow ActiveXObject instantiation failed (perhaps some special
        // security settings or otherwse), fall back to per-path storage
        storage = doc.createElement('div');
        storageOwner = doc.body;
      }
      forbiddenCharsRegex = new RegExp("[!\"#$%&'()*+,/\\\\:;<=>?@[\\]^`{|}~]", "g");

      store.set = withIEStorage(function (storage, key, val) {
        key = ieKeyFix(key);
        if (val === undefined) {
          return store.remove(key);
        }
        storage.setAttribute(key, store.serialize(val));
        storage.save(localStorageName);
        return val;
      });
      store.get = withIEStorage(function (storage, key) {
        key = ieKeyFix(key);
        return store.deserialize(storage.getAttribute(key));
      });
      store.remove = withIEStorage(function (storage, key) {
        key = ieKeyFix(key);
        storage.removeAttribute(key);
        storage.save(localStorageName);
      });
      store.clear = withIEStorage(function (storage) {
        var attributes = storage.XMLDocument.documentElement.attributes;
        storage.load(localStorageName);
        for (var i = 0, attr; attr = attributes[i]; i++) {
          storage.removeAttribute(attr.name);
        }
        storage.save(localStorageName);
      });
      store.getAll = withIEStorage(function (storage) {
        var attributes = storage.XMLDocument.documentElement.attributes;
        var ret = {};
        for (var i = 0, attr; attr = attributes[i]; ++i) {
          var key = ieKeyFix(attr.name);
          ret[attr.name] = store.deserialize(storage.getAttribute(key));
        }
        return ret;
      });
    })();
  }

  try {
    store.set(namespace, namespace);
    if (store.get(namespace) != namespace) {
      store.disabled = true;
    }
    store.remove(namespace);
  } catch (e) {
    store.disabled = true;
  }
  store.enabled = !store.disabled;
  if (typeof module != 'undefined' && module.exports) {
    module.exports = store;
  } else if (typeof define === 'function' && define.amd) {
    define(store);
  } else {
    this.store = store;
  }
})();

var __bind = function __bind(fn, me) {
  return function () {
    return fn.apply(me, arguments);
  };
},
    __slice = [].slice;

(function ($, window) {
  var ResizableColumns;

  ResizableColumns = (function () {
    ResizableColumns.prototype.defaults = {
      store: window.store,
      rigidSizing: false
    };

    function ResizableColumns($table, options) {
      this.mousedown = __bind(this.mousedown, this);
      var _this = this;

      this.options = $.extend({}, this.defaults, options);
      this.$table = $table;
      this.tableId = this.$table.data('resizable-columns-id');
      this.createHandles();
      this.restoreColumnWidths();
      this.syncHandleWidths();
      $(window).on('resize.rc', function () {
        return _this.syncHandleWidths();
      });
    }

    ResizableColumns.prototype.destroy = function () {
      this.$handleContainer.remove();
      this.$table.removeData('resizableColumns');
      return $(window).off('.rc');
    };

    ResizableColumns.prototype.createHandles = function () {
      var _this = this;

      this.$table.before(this.$handleContainer = $("<div class='rc-handle-container' />"));
      this.$table.find('tr th').each(function (i, el) {
        var $handle;

        if (_this.$table.find('tr th').eq(i + 1).length === 0 || _this.$table.find('tr th').eq(i).attr('data-noresize') != null || _this.$table.find('tr th').eq(i + 1).attr('data-noresize') != null) {
          return;
        }
        $handle = $("<div class='rc-handle' />");
        $handle.data('th', $(el));
        return $handle.appendTo(_this.$handleContainer);
      });
      return this.$handleContainer.on('mousedown', '.rc-handle', this.mousedown);
    };

    ResizableColumns.prototype.syncHandleWidths = function () {
      var _this = this;

      this.$handleContainer.width(this.$table.width());
      return this.$handleContainer.find('.rc-handle').each(function (_, el) {
        return $(el).css({
          left: $(el).data('th').outerWidth() + ($(el).data('th').offset().left - _this.$handleContainer.offset().left),
          height: _this.$table.height()
        });
      });
    };

    ResizableColumns.prototype.saveColumnWidths = function () {
      var _this = this;

      return this.$table.find('tr th').each(function (_, el) {
        var id;

        if ($(el).attr('data-noresize') == null) {
          id = _this.tableId + '-' + $(el).data('resizable-column-id');
          if (_this.options.store != null) {
            return store.set(id, $(el).width());
          }
        }
      });
    };

    ResizableColumns.prototype.restoreColumnWidths = function () {
      var _this = this;

      return this.$table.find('tr th').each(function (_, el) {
        var id, width;

        id = _this.tableId + '-' + $(el).data('resizable-column-id');
        if (_this.options.store != null && (width = store.get(id))) {
          return $(el).width(width);
        }
      });
    };

    ResizableColumns.prototype.mousedown = function (e) {
      var $currentGrip,
          $leftColumn,
          $rightColumn,
          idx,
          leftColumnStartWidth,
          rightColumnStartWidth,
          _this = this;

      e.preventDefault();
      this.startPosition = e.pageX;
      $currentGrip = $(e.currentTarget);
      $leftColumn = $currentGrip.data('th');
      leftColumnStartWidth = $leftColumn.width();
      idx = this.$table.find('tr th').index($currentGrip.data('th'));
      $rightColumn = this.$table.find('tr th').eq(idx + 1);
      rightColumnStartWidth = $rightColumn.width();
      $(document).on('mousemove.rc', function (e) {
        var difference, newLeftColumnWidth, newRightColumnWidth;

        difference = e.pageX - _this.startPosition;
        newRightColumnWidth = rightColumnStartWidth - difference;
        newLeftColumnWidth = leftColumnStartWidth + difference;
        if (_this.options.rigidSizing && parseInt($rightColumn[0].style.width) < $rightColumn.width() && newRightColumnWidth < $rightColumn.width() || parseInt($leftColumn[0].style.width) < $leftColumn.width() && newLeftColumnWidth < $leftColumn.width()) {
          return;
        }
        $leftColumn.width(newLeftColumnWidth);
        $rightColumn.width(newRightColumnWidth);
        return _this.syncHandleWidths();
      });
      return $(document).one('mouseup', function () {
        $(document).off('mousemove.rc');
        return _this.saveColumnWidths();
      });
    };

    return ResizableColumns;
  })();
  return $.fn.extend({
    resizableColumns: function resizableColumns() {
      var args, option;

      option = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      return this.each(function () {
        var $table, data;

        $table = $(this);
        data = $table.data('resizableColumns');
        if (!data) {
          $table.data('resizableColumns', data = new ResizableColumns($table, option));
        }
        if (typeof option === 'string') {
          return data[option].apply(data, args);
        }
      });
    }
  });
})(window.jQuery, window);